import { AnyShape, IssueOptions, Message, SetShape } from '../core';

declare module '../core' {
    interface SetShape<ValueShape extends AnyShape> {
        /**
         * Constrains the set size.
         *
         * @param size The minimum set size.
         * @param options The issue options or the issue message.
         * @returns The clone of the shape.
         * @group Plugin Methods
         * @plugin {@link plugin/set-essentials! plugin/set-essentials}
         */
        size(size: number, options?: IssueOptions | Message): this;
        /**
         * Constrains the minimum set size.
         *
         * @param size The minimum set size.
         * @param options The issue options or the issue message.
         * @returns The clone of the shape.
         * @group Plugin Methods
         * @plugin {@link plugin/set-essentials! plugin/set-essentials}
         */
        min(size: number, options?: IssueOptions | Message): this;
        /**
         * Constrains the maximum set size.
         *
         * @param size The maximum set size.
         * @param options The issue options or the issue message.
         * @returns The clone of the shape.
         * @group Plugin Methods
         * @plugin {@link plugin/set-essentials! plugin/set-essentials}
         */
        max(size: number, options?: IssueOptions | Message): this;
        /**
         * Constrains the {@link !Set Set} to contain at least one element.
         *
         * @param options The issue options or the issue message.
         * @returns The clone of the shape.
         * @group Plugin Methods
         * @plugin {@link plugin/set-essentials! plugin/set-essentials}
         */
        nonEmpty(options?: IssueOptions | Message): this;
    }
}
/**
 * Enhances {@link core!SetShape SetShape} with additional methods.
 */
declare function enableSetEssentials(prototype: SetShape<any>): void;

export { enableSetEssentials as default };
