import { IssueOptions, Message, ApplyOptions, Issue } from './core';

/**
 * The module with the utility functions that can be used for plugin development.
 *
 * ```ts
 * import { createIssueFactory } from 'doubter/utils';
 * ```
 *
 * @module utils
 */

/**
 * Extracts options from the given source.
 *
 * @param source Options or message to extract from.
 */
declare function extractOptions<T extends IssueOptions>(source: T | Message | undefined): Partial<T>;
/**
 * Returns a function that creates an issue.
 *
 * @param code The code of the issue.
 * @param defaultMessage The default message that is used if message isn't provided through options.
 * @param options The issue options or the issue message.
 * @param param The param that is added to the issue.
 * @returns The callback that takes an input and options, and returns an issue.
 */
declare function createIssueFactory(code: unknown, defaultMessage: unknown, options: IssueOptions | Message | undefined, param: unknown): (input: unknown, options: ApplyOptions) => Issue;
/**
 * Returns a function that creates an issue.
 *
 * @param code The code of the issue.
 * @param defaultMessage The default message that is used if message isn't provided through options.
 * @param options The issue options or the issue message.
 * @returns The callback that takes an input, options, and a param, and returns an issue.
 */
declare function createIssueFactory(code: unknown, defaultMessage: unknown, options: IssueOptions | Message | undefined): (input: unknown, options: ApplyOptions, param: unknown) => Issue;

export { createIssueFactory, extractOptions };
