'use strict';

function isObjectLike(value) {
    return value !== null && typeof value === 'object';
}

/**
 * The module with the utility functions that can be used for plugin development.
 *
 * ```ts
 * import { createIssueFactory } from 'doubter/utils';
 * ```
 *
 * @module utils
 */
/**
 * Extracts options from the given source.
 *
 * @param source Options or message to extract from.
 */
function extractOptions(source) {
    if (typeof source === 'function' || typeof source === 'string') {
        return { message: source };
    }
    if (isObjectLike(source)) {
        return source;
    }
    return {};
}
function createIssueFactory(code, defaultMessage, options, param) {
    var parameterized = arguments.length <= 3;
    var _a = extractOptions(options), _b = _a.message, message = _b === void 0 ? defaultMessage : _b, meta = _a.meta;
    if (typeof message === 'function') {
        return function (input, options, param0) {
            var issue = { code: code, path: undefined, input: input, message: undefined, param: parameterized ? param0 : param, meta: meta };
            var value = message(issue, options);
            if (issue.message === undefined) {
                issue.message = value;
            }
            return issue;
        };
    }
    if (typeof message === 'string') {
        if (parameterized) {
            if (message.indexOf('%s') !== -1) {
                return function (input, options, param0) {
                    return { code: code, path: undefined, input: input, message: message.replace('%s', String(param0)), param: param0, meta: meta };
                };
            }
        }
        else {
            message = message.replace('%s', String(param));
        }
    }
    if (parameterized) {
        return function (input, options, param0) {
            return { code: code, path: undefined, input: input, message: message, param: param0, meta: meta };
        };
    }
    return function (input, options) {
        return { code: code, path: undefined, input: input, message: message, param: param, meta: meta };
    };
}

exports.createIssueFactory = createIssueFactory;
exports.extractOptions = extractOptions;
